// -*- C++ -*-
/*!
 * @file  Constant.cpp
 * @brief Constant component
 * $Date$
 *
 * $Id$
 */

#include "Constant.h"

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4

// Module specification
// <rtc-template block="module_spec">
static const char* constant_spec[] =
  {
    "implementation_id", "Constant",
    "type_name",         "Constant",
    "description",       "Constant component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.Output", "10",
    "conf.default.Start_Flag", "0",

    ""
  };
// </rtc-template>

Constant::Constant(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("constant", "constant", m_constant);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Constant::~Constant()
{
}


RTC::ReturnCode_t Constant::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("Output", m_Output, "10");
  bindParameter("Start_Flag", m_Start_Flag, "0");
  
  // </rtc-template>
  std::cout << "Constant start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t Constant::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}


bool svc_flag=false;
int svc_type=0;
double svc_out=0;

bool activate_flag=true;
bool execute_flag=false;

RTC::ReturnCode_t Constant::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
  }


  switch(type){
    case SHORT:
      registerOutPort("shortout", m_shortoutOut);
    break;
    case LONG:
        registerOutPort("longout", m_longoutOut);
    break;
    case FLOAT:
        registerOutPort("floatout", m_floatoutOut);
    break;
    case DOUBLE:
        registerOutPort("doubleout", m_doubleoutOut);
    break;
  }

  activate_flag=false;
  }

  if(svc_flag)
    out = svc_out;
  else
    out = m_Output;
    svc_out = out;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Constant::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



RTC::ReturnCode_t Constant::onExecute(RTC::UniqueId ec_id)
{
  bool start_flag=false;

  if(svc_flag)
    start_flag = execute_flag;
  else
    start_flag = m_Start_Flag;

  if(start_flag){
    if(svc_flag)
      out = svc_out;
    else
      out = m_Output;
      svc_out = out;

    switch(type){
      case SHORT:
        m_shortout.data = (short)out;
        m_shortoutOut.write();
      break;

      case LONG:
        m_longout.data = (short)out;
        m_longoutOut.write();
      break;

      case FLOAT:
        m_floatout.data = (float)out;
        m_floatoutOut.write();
      break;

      case DOUBLE:
        m_doubleout.data = out;
        m_doubleoutOut.write();
      break;
    }
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Constant::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Constant::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Constant::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Constant::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Constant::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ConstantInit(RTC::Manager* manager)
  {
    RTC::Properties profile(constant_spec);
    manager->registerFactory(profile,
                             RTC::Create<Constant>,
                             RTC::Delete<Constant>);
  }
  
};


