// -*- C++ -*-
/*!
 * @file  Duplicator.h
 * @brief Duplicator component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef DUPLICATOR_H
#define DUPLICATOR_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <string>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "DuplicatorSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class Duplicator
  : public RTC::DataFlowComponentBase
{
 public:
  Duplicator(RTC::Manager* manager);
  ~Duplicator();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_DataType;
  int m_OutPort_Num;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedLong m_longin;
  InPort<TimedLong> m_longinIn;
  TimedShort m_shortin;
  InPort<TimedShort> m_shortinIn;
  TimedFloat m_floatin;
  InPort<TimedFloat> m_floatinIn;
  TimedDouble m_doublein;
  InPort<TimedDouble> m_doubleinIn;
  TimedBoolean m_boolin;
  InPort<TimedBoolean> m_boolinIn;

  TimedLongSeq m_larrin;
  InPort<TimedLongSeq> m_larrinIn;
  TimedShortSeq m_sarrin;
  InPort<TimedShortSeq> m_sarrinIn;
  TimedFloatSeq m_farrin;
  InPort<TimedFloatSeq> m_farrinIn;
  TimedDoubleSeq m_darrin;
  InPort<TimedDoubleSeq> m_darrinIn;
  TimedBooleanSeq m_barrin;
  InPort<TimedBooleanSeq> m_barrinIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedLong m_longout0;
  OutPort<TimedLong> m_longout0Out;
  TimedLong m_longout1;
  OutPort<TimedLong> m_longout1Out;
  TimedLong m_longout2;
  OutPort<TimedLong> m_longout2Out;
  TimedLong m_longout3;
  OutPort<TimedLong> m_longout3Out;
  TimedLong m_longout4;
  OutPort<TimedLong> m_longout4Out;
  TimedLong m_longout5;
  OutPort<TimedLong> m_longout5Out;
  TimedLong m_longout6;
  OutPort<TimedLong> m_longout6Out;
  TimedLong m_longout7;
  OutPort<TimedLong> m_longout7Out;
  TimedLong m_longout8;
  OutPort<TimedLong> m_longout8Out;
  TimedLong m_longout9;
  OutPort<TimedLong> m_longout9Out;

  TimedShort m_shortout0;
  OutPort<TimedShort> m_shortout0Out;
  TimedShort m_shortout1;
  OutPort<TimedShort> m_shortout1Out;
  TimedShort m_shortout2;
  OutPort<TimedShort> m_shortout2Out;
  TimedShort m_shortout3;
  OutPort<TimedShort> m_shortout3Out;
  TimedShort m_shortout4;
  OutPort<TimedShort> m_shortout4Out;
  TimedShort m_shortout5;
  OutPort<TimedShort> m_shortout5Out;
  TimedShort m_shortout6;
  OutPort<TimedShort> m_shortout6Out;
  TimedShort m_shortout7;
  OutPort<TimedShort> m_shortout7Out;
  TimedShort m_shortout8;
  OutPort<TimedShort> m_shortout8Out;
  TimedShort m_shortout9;
  OutPort<TimedShort> m_shortout9Out;

  TimedFloat m_floatout0;
  OutPort<TimedFloat> m_floatout0Out;
  TimedFloat m_floatout1;
  OutPort<TimedFloat> m_floatout1Out;
  TimedFloat m_floatout2;
  OutPort<TimedFloat> m_floatout2Out;
  TimedFloat m_floatout3;
  OutPort<TimedFloat> m_floatout3Out;
  TimedFloat m_floatout4;
  OutPort<TimedFloat> m_floatout4Out;
  TimedFloat m_floatout5;
  OutPort<TimedFloat> m_floatout5Out;
  TimedFloat m_floatout6;
  OutPort<TimedFloat> m_floatout6Out;
  TimedFloat m_floatout7;
  OutPort<TimedFloat> m_floatout7Out;
  TimedFloat m_floatout8;
  OutPort<TimedFloat> m_floatout8Out;
  TimedFloat m_floatout9;
  OutPort<TimedFloat> m_floatout9Out;

  TimedDouble m_doubleout0;
  OutPort<TimedDouble> m_doubleout0Out;
  TimedDouble m_doubleout1;
  OutPort<TimedDouble> m_doubleout1Out;
  TimedDouble m_doubleout2;
  OutPort<TimedDouble> m_doubleout2Out;
  TimedDouble m_doubleout3;
  OutPort<TimedDouble> m_doubleout3Out;
  TimedDouble m_doubleout4;
  OutPort<TimedDouble> m_doubleout4Out;
  TimedDouble m_doubleout5;
  OutPort<TimedDouble> m_doubleout5Out;
  TimedDouble m_doubleout6;
  OutPort<TimedDouble> m_doubleout6Out;
  TimedDouble m_doubleout7;
  OutPort<TimedDouble> m_doubleout7Out;
  TimedDouble m_doubleout8;
  OutPort<TimedDouble> m_doubleout8Out;
  TimedDouble m_doubleout9;
  OutPort<TimedDouble> m_doubleout9Out;

  TimedBoolean m_boolout0;
  OutPort<TimedBoolean> m_boolout0Out;
  TimedBoolean m_boolout1;
  OutPort<TimedBoolean> m_boolout1Out;
  TimedBoolean m_boolout2;
  OutPort<TimedBoolean> m_boolout2Out;
  TimedBoolean m_boolout3;
  OutPort<TimedBoolean> m_boolout3Out;
  TimedBoolean m_boolout4;
  OutPort<TimedBoolean> m_boolout4Out;
  TimedBoolean m_boolout5;
  OutPort<TimedBoolean> m_boolout5Out;
  TimedBoolean m_boolout6;
  OutPort<TimedBoolean> m_boolout6Out;
  TimedBoolean m_boolout7;
  OutPort<TimedBoolean> m_boolout7Out;
  TimedBoolean m_boolout8;
  OutPort<TimedBoolean> m_boolout8Out;
  TimedBoolean m_boolout9;
  OutPort<TimedBoolean> m_boolout9Out;

  TimedLongSeq m_larrout0;
  OutPort<TimedLongSeq> m_larrout0Out;
  TimedLongSeq m_larrout1;
  OutPort<TimedLongSeq> m_larrout1Out;
  TimedLongSeq m_larrout2;
  OutPort<TimedLongSeq> m_larrout2Out;
  TimedLongSeq m_larrout3;
  OutPort<TimedLongSeq> m_larrout3Out;
  TimedLongSeq m_larrout4;
  OutPort<TimedLongSeq> m_larrout4Out;
  TimedLongSeq m_larrout5;
  OutPort<TimedLongSeq> m_larrout5Out;
  TimedLongSeq m_larrout6;
  OutPort<TimedLongSeq> m_larrout6Out;
  TimedLongSeq m_larrout7;
  OutPort<TimedLongSeq> m_larrout7Out;
  TimedLongSeq m_larrout8;
  OutPort<TimedLongSeq> m_larrout8Out;
  TimedLongSeq m_larrout9;
  OutPort<TimedLongSeq> m_larrout9Out;

  TimedShortSeq m_sarrout0;
  OutPort<TimedShortSeq> m_sarrout0Out;
  TimedShortSeq m_sarrout1;
  OutPort<TimedShortSeq> m_sarrout1Out;
  TimedShortSeq m_sarrout2;
  OutPort<TimedShortSeq> m_sarrout2Out;
  TimedShortSeq m_sarrout3;
  OutPort<TimedShortSeq> m_sarrout3Out;
  TimedShortSeq m_sarrout4;
  OutPort<TimedShortSeq> m_sarrout4Out;
  TimedShortSeq m_sarrout5;
  OutPort<TimedShortSeq> m_sarrout5Out;
  TimedShortSeq m_sarrout6;
  OutPort<TimedShortSeq> m_sarrout6Out;
  TimedShortSeq m_sarrout7;
  OutPort<TimedShortSeq> m_sarrout7Out;
  TimedShortSeq m_sarrout8;
  OutPort<TimedShortSeq> m_sarrout8Out;
  TimedShortSeq m_sarrout9;
  OutPort<TimedShortSeq> m_sarrout9Out;

  TimedFloatSeq m_farrout0;
  OutPort<TimedFloatSeq> m_farrout0Out;
  TimedFloatSeq m_farrout1;
  OutPort<TimedFloatSeq> m_farrout1Out;
  TimedFloatSeq m_farrout2;
  OutPort<TimedFloatSeq> m_farrout2Out;
  TimedFloatSeq m_farrout3;
  OutPort<TimedFloatSeq> m_farrout3Out;
  TimedFloatSeq m_farrout4;
  OutPort<TimedFloatSeq> m_farrout4Out;
  TimedFloatSeq m_farrout5;
  OutPort<TimedFloatSeq> m_farrout5Out;
  TimedFloatSeq m_farrout6;
  OutPort<TimedFloatSeq> m_farrout6Out;
  TimedFloatSeq m_farrout7;
  OutPort<TimedFloatSeq> m_farrout7Out;
  TimedFloatSeq m_farrout8;
  OutPort<TimedFloatSeq> m_farrout8Out;
  TimedFloatSeq m_farrout9;
  OutPort<TimedFloatSeq> m_farrout9Out;

  TimedDoubleSeq m_darrout0;
  OutPort<TimedDoubleSeq> m_darrout0Out;
  TimedDoubleSeq m_darrout1;
  OutPort<TimedDoubleSeq> m_darrout1Out;
  TimedDoubleSeq m_darrout2;
  OutPort<TimedDoubleSeq> m_darrout2Out;
  TimedDoubleSeq m_darrout3;
  OutPort<TimedDoubleSeq> m_darrout3Out;
  TimedDoubleSeq m_darrout4;
  OutPort<TimedDoubleSeq> m_darrout4Out;
  TimedDoubleSeq m_darrout5;
  OutPort<TimedDoubleSeq> m_darrout5Out;
  TimedDoubleSeq m_darrout6;
  OutPort<TimedDoubleSeq> m_darrout6Out;
  TimedDoubleSeq m_darrout7;
  OutPort<TimedDoubleSeq> m_darrout7Out;
  TimedDoubleSeq m_darrout8;
  OutPort<TimedDoubleSeq> m_darrout8Out;
  TimedDoubleSeq m_darrout9;
  OutPort<TimedDoubleSeq> m_darrout9Out;

  TimedBooleanSeq m_barrout0;
  OutPort<TimedBooleanSeq> m_barrout0Out;
  TimedBooleanSeq m_barrout1;
  OutPort<TimedBooleanSeq> m_barrout1Out;
  TimedBooleanSeq m_barrout2;
  OutPort<TimedBooleanSeq> m_barrout2Out;
  TimedBooleanSeq m_barrout3;
  OutPort<TimedBooleanSeq> m_barrout3Out;
  TimedBooleanSeq m_barrout4;
  OutPort<TimedBooleanSeq> m_barrout4Out;
  TimedBooleanSeq m_barrout5;
  OutPort<TimedBooleanSeq> m_barrout5Out;
  TimedBooleanSeq m_barrout6;
  OutPort<TimedBooleanSeq> m_barrout6Out;
  TimedBooleanSeq m_barrout7;
  OutPort<TimedBooleanSeq> m_barrout7Out;
  TimedBooleanSeq m_barrout8;
  OutPort<TimedBooleanSeq> m_barrout8Out;
  TimedBooleanSeq m_barrout9;
  OutPort<TimedBooleanSeq> m_barrout9Out;
  
  int num, type, element;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_PrvPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  duplicatorSVC_impl m_duplicator;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

};


extern "C"
{
  void DuplicatorInit(RTC::Manager* manager);
};

#endif // DUPLICATOR_H
