// -*- C++ -*-
/*!
 * @file  FileWriter.cpp
 * @brief FileWriter component
 * $Date$
 *
 * $Id$
 */

#include "FileWriter.h"
#include <iostream>
#include <fstream>
#include <iomanip>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40

// Module specification
// <rtc-template block="module_spec">
static const char* filewriter_spec[] =
  {
    "implementation_id", "FileWriter",
    "type_name",         "FileWriter",
    "description",       "FileWriter component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.FileName", "RTMdata",
    "conf.default.Store_Time", "10",
    "conf.default.Start_Flag", "0",
    "conf.default.ART_Cycle", "0.001",

    ""
  };
// </rtc-template>

FileWriter::FileWriter(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortinIn("shortin", m_shortin),
    m_longinIn("longin", m_longin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),

    m_sarrinIn("shortarrayin", m_sarrin),
    m_larrinIn("longarrayin", m_larrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("filewriter", "filewriter", m_filewriter);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

FileWriter::~FileWriter()
{
}


RTC::ReturnCode_t FileWriter::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("FileName", m_FileName, "RTMdata");
  bindParameter("Store_Time", m_Store_Time, "10");
  bindParameter("Start_Flag", m_Start_Flag, "0");
  bindParameter("ART_Cycle", m_ART_Cycle, "0.001");
  
  // </rtc-template>
  std::cout << "FileWriter start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileWriter::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}


bool svc_flag=false;
bool svcname_flag=false;
int svc_type=0;
std::string svc_filename;
double svc_time=0.1;
bool svc_start=false;
bool activate_flag=true;

RTC::ReturnCode_t FileWriter::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){
    if(svc_flag)
      type = svc_type;
    else{
      if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray"){
        std::cout << "InPort_DataType ERROR" << std::endl;
        return RTC::RTC_ERROR;
      }
      if(m_DataType=="short")
        type = SHORT;
      else if(m_DataType=="long")
        type = LONG;
      else if(m_DataType=="float")
        type = FLOAT;
      else if(m_DataType=="double")
        type = DOUBLE;
      else if(m_DataType=="shortArray")
        type = SHORTARRAY;
      else if(m_DataType=="longArray")
        type = LONGARRAY;
      else if(m_DataType=="floatArray")
        type = FLOATARRAY;
      else if(m_DataType=="doubleArray")
        type = DOUBLEARRAY;
    }

    switch(type){
      case SHORT:
        registerInPort("shortin", m_shortinIn);
      break;
      case LONG:
        registerInPort("longin", m_longinIn);
      break;
      case FLOAT:
        registerInPort("floatin", m_floatinIn);
      break;
      case DOUBLE:
        registerInPort("doublein", m_doubleinIn);
      break;
      case SHORTARRAY:
        registerInPort("shortArrayin", m_sarrinIn);
      break;
      case LONGARRAY:
        registerInPort("longArrayin", m_larrinIn);
      break;
      case FLOATARRAY:
        registerInPort("floatArrayin", m_farrinIn);
      break;
      case DOUBLEARRAY:
        registerInPort("doubleArrayin", m_darrinIn);
      break;
    }

    activate_flag=false;
  }

  return RTC::RTC_OK;
}

std::vector<bool> inflag;
std::vector<short> shortnum;
std::vector<long> longnum;
std::vector<float> floatnum;
std::vector<double> doublenum;
std::vector< std::vector<short> > sarrnum;
std::vector< std::vector<long> > larrnum;
std::vector< std::vector<float> > farrnum;
std::vector< std::vector<double> > darrnum;
bool start_flag=false;
bool store_flag=false;
bool firstpushback_flag=true;


RTC::ReturnCode_t FileWriter::onDeactivated(RTC::UniqueId ec_id)
{
  inflag.clear();
  shortnum.clear();
  longnum.clear();
  floatnum.clear();
  doublenum.clear();
  sarrnum.clear();
  larrnum.clear();
  farrnum.clear();
  darrnum.clear();
  start_flag=false;
  store_flag=false;
  firstpushback_flag=true;
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileWriter::onExecute(RTC::UniqueId ec_id)
{
  if(svc_flag)
    start_flag = svc_start;
  else
    start_flag = (bool)m_Start_Flag;

  if(start_flag){
    store_flag = true;

    switch(type){
      case SHORT:
        if(m_shortinIn.isNew()){
          m_shortinIn.read();
          shortnum.push_back(m_shortin.data);
          inflag.push_back(true);
        }
        else{
          shortnum.push_back(0);
          inflag.push_back(false);
        }
      break;

      case LONG:
        if(m_longinIn.isNew()){
          m_longinIn.read();
          longnum.push_back(m_longin.data);
          inflag.push_back(true);
        }
        else{
          longnum.push_back(0);
          inflag.push_back(false);
        }
      break;

      case FLOAT:
        if(m_floatinIn.isNew()){
          m_floatinIn.read();
          floatnum.push_back(m_floatin.data);
          inflag.push_back(true);
        }
        else{
          floatnum.push_back(0);
          inflag.push_back(false);
        }
      break;

      case DOUBLE:
        if(m_doubleinIn.isNew()){
          m_doubleinIn.read();
          doublenum.push_back(m_doublein.data);
          inflag.push_back(true);
        }
        else{
          doublenum.push_back(0);
          inflag.push_back(false);
        }
      break;

      case SHORTARRAY:
        if(m_sarrinIn.isNew()){
          m_sarrinIn.read();
          if(firstpushback_flag){
            for(int i=0, len=m_sarrin.data.length(); i<len; i++)
              sarrnum.push_back(shortnum);
            firstpushback_flag = false;
          }
          for(int i=0, len=m_sarrin.data.length(); i<len; i++)
            sarrnum[i].push_back(m_sarrin.data[i]);
          inflag.push_back(true);
        }
        else{
          for(int i=0, len=m_sarrin.data.length(); i<len; i++)
            sarrnum[i].push_back(0);
          inflag.push_back(false);
        }
      break;

      case LONGARRAY:
        if(m_larrinIn.isNew()){
          m_larrinIn.read();
          if(firstpushback_flag){
            for(int i=0, len=m_larrin.data.length(); i<len; i++)
              larrnum.push_back(longnum);
            firstpushback_flag = false;
          }
          for(int i=0, len=m_larrin.data.length(); i<len; i++)
            larrnum[i].push_back(m_larrin.data[i]);
          inflag.push_back(true);
        }
        else{
          for(int i=0, len=m_larrin.data.length(); i<len; i++)
            larrnum[i].push_back(0);
          inflag.push_back(false);
        }
      break;

      case FLOATARRAY:
        if(m_farrinIn.isNew()){
          m_farrinIn.read();
          if(firstpushback_flag){
            for(int i=0, len=m_farrin.data.length(); i<len; i++)
              farrnum.push_back(floatnum);
            firstpushback_flag = false;
          }
          for(int i=0, len=m_farrin.data.length(); i<len; i++)
            farrnum[i].push_back(m_farrin.data[i]);
          inflag.push_back(true);
        }
        else{
          for(int i=0, len=m_farrin.data.length(); i<len; i++)
            farrnum[i].push_back(0);
          inflag.push_back(false);
        }
      break;

      case DOUBLEARRAY:
        if(m_darrinIn.isNew()){
          m_darrinIn.read();
          if(firstpushback_flag){
            for(int i=0, len=m_darrin.data.length(); i<len; i++)
              darrnum.push_back(doublenum);
            firstpushback_flag = false;
          }
          for(int i=0, len=m_darrin.data.length(); i<len; i++)
            darrnum[i].push_back(m_darrin.data[i]);
          inflag.push_back(true);
        }
        else{
          for(int i=0, len=m_darrin.data.length(); i<len; i++)
            darrnum[i].push_back(0);
          inflag.push_back(false);
        }
      break;
    }
    comptime++;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileWriter::onStateUpdate(RTC::UniqueId ec_id)
{
  double storetime;
  std::string name;
  static int filename_count=0;

  if(svc_flag)
    storetime = svc_time;
  else
    storetime = m_Store_Time;

  if(storetime <= 0){
    std::cout << "Store_Time ERROR" << std::endl;
    return RTC::RTC_ERROR;
  }

  if((comptime >= storetime/m_ART_Cycle)||((store_flag)&&(start_flag==false))){
    if(svcname_flag)
      filename = svc_filename;
    else
      filename = m_FileName;

    name = filename + otos(filename_count) + ".dat";

    std::ofstream fout(name.c_str());
    if(!fout){
      std::cout << "File can't open!" << std::endl;
      return RTC::RTC_ERROR;
    }
    else
      std::cout << "fileopen." << std::endl;

    switch(type){
      case SHORT:
        for(int i=0; i<comptime; i++){
          if(inflag[i])
            fout << std::setw(8) << i << std::setw(8) << shortnum[i] << std::endl;
        }
      break;

      case LONG:
        for(int i=0; i<comptime; i++){
          if(inflag[i])
            fout << std::setw(8) << i << std::setw(15) << longnum[i] << std::endl;
        }
      break;

      case FLOAT:
        for(int i=0; i<comptime; i++){
          if(inflag[i])
            fout << std::setw(8) << i << std::setw(15) << floatnum[i] << std::endl;
        }
      break;

      case DOUBLE:
        for(int i=0; i<comptime; i++){
          if(inflag[i])
            fout << std::setw(8) << i << std::setw(15) << doublenum[i] << std::endl;
        }
      break;

      case SHORTARRAY:
        for(int i=0; i<comptime; i++){
          if(inflag[i]){
            fout << std::setw(8) << i;
            for(int j=0; j<sarrnum.size(); j++)
              fout << std::setw(15) << sarrnum[j][i];
            fout << std::endl;
          }
        }
      break;

      case LONGARRAY:
        for(int i=0; i<comptime; i++){
          if(inflag[i]){
            fout << std::setw(8) << i;
            for(int j=0; j<larrnum.size(); j++)
              fout << std::setw(15) << larrnum[j][i];
            fout << std::endl;
          }
        }
      break;

      case FLOATARRAY:
        for(int i=0; i<comptime; i++){
          if(inflag[i]){
            fout << std::setw(8) << i;
            for(int j=0; j<farrnum.size(); j++)
              fout << std::setw(15) << farrnum[j][i];
            fout << std::endl;
          }
        }
      break;

      case DOUBLEARRAY:
        for(int i=0; i<comptime; i++){
          if(inflag[i]){
            fout << std::setw(8) << i;
            for(int j=0; j<darrnum.size(); j++)
              fout << std::setw(15) << darrnum[j][i];
            fout << std::endl;
          }
        }
      break;
    }

    fout.close();

    std::cout << "Data store finish. " << name << " is made." << std::endl;
    filename_count++;


    comptime = 0;
    start_flag = false;
    store_flag = false;
    svc_start = false;
    m_Start_Flag = 0;
    inflag.clear();
    shortnum.clear();
    longnum.clear();
    floatnum.clear();
    doublenum.clear();
    sarrnum.clear();
    larrnum.clear();
    farrnum.clear();
    darrnum.clear();
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t FileWriter::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileWriter::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileWriter::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileWriter::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FileWriterInit(RTC::Manager* manager)
  {
    RTC::Properties profile(filewriter_spec);
    manager->registerFactory(profile,
                             RTC::Create<FileWriter>,
                             RTC::Delete<FileWriter>);
  }
  
};


