// -*- C++ -*-
/*!
 * @file  Integrator.cpp
 * @brief Integrator component
 * $Date$
 *
 * $Id$
 */

#include "Integrator.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40

// Module specification
// <rtc-template block="module_spec">
static const char* integrator_spec[] =
  {
    "implementation_id", "Integrator",
    "type_name",         "Integrator",
    "description",       "Integrator component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.ART_Cycle", "0.001",

    ""
  };
// </rtc-template>

Integrator::Integrator(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortinIn("shortin", m_shortin),
    m_longinIn("longin", m_longin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),

    m_sarrinIn("shortarrayin", m_sarrin),
    m_larrinIn("longarrayin", m_larrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),

    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  //registerInPort("dataout", m_dataoutIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("integrator", "integrator", m_integrator);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Integrator::~Integrator()
{
}


RTC::ReturnCode_t Integrator::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("ART_Cycle", m_ART_Cycle, "0.001");
  
  // </rtc-template>
  std::cout << "Integrator start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t Integrator::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}


bool svc_flag=false;
int svc_type=0;
bool activate_flag=true;

RTC::ReturnCode_t Integrator::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
    else if(m_DataType=="shortArray")
      type = SHORTARRAY;
    else if(m_DataType=="longArray")
      type = LONGARRAY;
    else if(m_DataType=="floatArray")
      type = FLOATARRAY;
    else if(m_DataType=="doubleArray")
      type = DOUBLEARRAY;
  }


  switch(type){
    case SHORT:
      registerInPort("shortin", m_shortinIn);
      registerOutPort("shortout", m_shortoutOut);
    break;
    case LONG:
      registerInPort("longin", m_longinIn);
      registerOutPort("longout", m_longoutOut);
    break;
    case FLOAT:
      registerInPort("floatin", m_floatinIn);
      registerOutPort("floatout", m_floatoutOut);
    break;
    case DOUBLE:
      registerInPort("doublein", m_doubleinIn);
      registerOutPort("doubleout", m_doubleoutOut);
    break;
    case SHORTARRAY:
      registerInPort("shortArrayin", m_sarrinIn);
      registerOutPort("shortArrayout", m_sarroutOut);
    break;
    case LONGARRAY:
      registerInPort("longArrayin", m_larrinIn);
      registerOutPort("longArrayout", m_larroutOut);
    break;
    case FLOATARRAY:
      registerInPort("floatArrayin", m_farrinIn);
      registerOutPort("floatArrayout", m_farroutOut);
    break;
    case DOUBLEARRAY:
      registerInPort("doubleArrayin", m_darrinIn);
      registerOutPort("doubleArrayout", m_darroutOut);
    break;
  }


  activate_flag=false;
  }

  return RTC::RTC_OK;
}


bool firstpushback_flag=true;
short shortnum = 0;
long longnum = 0;
float floatnum = 0;
double doublenum = 0;
std::vector<short> sarrnum;
std::vector<long> larrnum;
std::vector<float> farrnum;
std::vector<double> darrnum;

RTC::ReturnCode_t Integrator::onDeactivated(RTC::UniqueId ec_id)
{
  firstpushback_flag=true;
  shortnum = 0;
  longnum = 0;
  floatnum = 0;
  doublenum = 0;
  sarrnum.clear();
  larrnum.clear();
  farrnum.clear();
  darrnum.clear();
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Integrator::onExecute(RTC::UniqueId ec_id)
{
  switch(type){
    case SHORT:
      if(m_shortinIn.isNew()){
        m_shortinIn.read();
        shortnum += m_shortin.data;
        m_shortout.data = (short)(shortnum * m_ART_Cycle);
        m_shortoutOut.write();
      }
    break;

    case LONG:
      if(m_longinIn.isNew()){
        m_longinIn.read();
        longnum += m_longin.data;
        m_longout.data = (long)(longnum * m_ART_Cycle);
        m_longoutOut.write();
      }
    break;

    case FLOAT:
      if(m_floatinIn.isNew()){
        m_floatinIn.read();
        floatnum += m_floatin.data;
        m_floatout.data = (float)(floatnum * m_ART_Cycle);
        m_floatoutOut.write();
      }
    break;

    case DOUBLE:
      if(m_doubleinIn.isNew()){
        m_doubleinIn.read();
        doublenum += m_doublein.data;
        m_doubleout.data = doublenum * m_ART_Cycle;
        m_doubleoutOut.write();
      }
    break;

    case SHORTARRAY:
      if(m_sarrinIn.isNew()){
        m_sarrinIn.read();
        if(firstpushback_flag){
          for(int i=0, len=m_sarrin.data.length(); i<len; i++)
            sarrnum.push_back(0);
          firstpushback_flag=false;
        }
        for(int i=0, len=m_sarrin.data.length(); i<len; i++)
          sarrnum[i] += m_sarrin.data[i];
        m_sarrout.data.length(m_sarrin.data.length());
        for(int i=0, len=m_sarrin.data.length(); i<len; i++)
          m_sarrout.data[i] = (short)(sarrnum[i] * m_ART_Cycle);
        m_sarroutOut.write();
      }
    break;

    case LONGARRAY:
      if(m_larrinIn.isNew()){
        m_larrinIn.read();
        if(firstpushback_flag){
          for(int i=0, len=m_larrin.data.length(); i<len; i++)
            larrnum.push_back(0);
          firstpushback_flag=false;
        }
        for(int i=0, len=m_larrin.data.length(); i<len; i++)
          larrnum[i] += m_larrin.data[i];
        m_larrout.data.length(m_larrin.data.length());
        for(int i=0, len=m_larrin.data.length(); i<len; i++)
          m_larrout.data[i] = (long)(larrnum[i] * m_ART_Cycle);
        m_larroutOut.write();
      }
    break;

    case FLOATARRAY:
      if(m_farrinIn.isNew()){
        m_farrinIn.read();
        if(firstpushback_flag){
          for(int i=0, len=m_farrin.data.length(); i<len; i++)
            farrnum.push_back(0);
          firstpushback_flag=false;
        }
        for(int i=0, len=m_farrin.data.length(); i<len; i++)
          farrnum[i] += m_farrin.data[i];
        m_farrout.data.length(m_farrin.data.length());
        for(int i=0, len=m_farrin.data.length(); i<len; i++)
          m_farrout.data[i] = (float)(farrnum[i] * m_ART_Cycle);
        m_farroutOut.write();
      }
    break;

    case DOUBLEARRAY:
      if(m_darrinIn.isNew()){
        m_darrinIn.read();
        if(firstpushback_flag){
          for(int i=0, len=m_darrin.data.length(); i<len; i++)
            darrnum.push_back(0);
          firstpushback_flag=false;
        }
        for(int i=0, len=m_darrin.data.length(); i<len; i++)
          darrnum[i] += m_darrin.data[i];
        m_darrout.data.length(m_darrin.data.length());
        for(int i=0, len=m_darrin.data.length(); i<len; i++)
          m_darrout.data[i] = darrnum[i] * m_ART_Cycle;
        m_darroutOut.write();
      }
    break;
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Integrator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Integrator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Integrator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Integrator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Integrator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void IntegratorInit(RTC::Manager* manager)
  {
    RTC::Properties profile(integrator_spec);
    manager->registerFactory(profile,
                             RTC::Create<Integrator>,
                             RTC::Delete<Integrator>);
  }
  
};


