// -*- C++ -*-
/*!
 * @file  Multiplexer.h
 * @brief Multiplexer component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef MULTIPLEXER_H
#define MULTIPLEXER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "MultiplexerSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class Multiplexer
  : public RTC::DataFlowComponentBase
{
 public:
  Multiplexer(RTC::Manager* manager);
  ~Multiplexer();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_DataType;
  int m_InPort_Num;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedShort m_shortin0;
  InPort<TimedShort> m_shortin0In;
  TimedShort m_shortin1;
  InPort<TimedShort> m_shortin1In;
  TimedShort m_shortin2;
  InPort<TimedShort> m_shortin2In;
  TimedShort m_shortin3;
  InPort<TimedShort> m_shortin3In;
  TimedShort m_shortin4;
  InPort<TimedShort> m_shortin4In;
  TimedShort m_shortin5;
  InPort<TimedShort> m_shortin5In;
  TimedShort m_shortin6;
  InPort<TimedShort> m_shortin6In;
  TimedShort m_shortin7;
  InPort<TimedShort> m_shortin7In;
  TimedShort m_shortin8;
  InPort<TimedShort> m_shortin8In;
  TimedShort m_shortin9;
  InPort<TimedShort> m_shortin9In;

  TimedLong m_longin0;
  InPort<TimedLong> m_longin0In;
  TimedLong m_longin1;
  InPort<TimedLong> m_longin1In;
  TimedLong m_longin2;
  InPort<TimedLong> m_longin2In;
  TimedLong m_longin3;
  InPort<TimedLong> m_longin3In;
  TimedLong m_longin4;
  InPort<TimedLong> m_longin4In;
  TimedLong m_longin5;
  InPort<TimedLong> m_longin5In;
  TimedLong m_longin6;
  InPort<TimedLong> m_longin6In;
  TimedLong m_longin7;
  InPort<TimedLong> m_longin7In;
  TimedLong m_longin8;
  InPort<TimedLong> m_longin8In;
  TimedLong m_longin9;
  InPort<TimedLong> m_longin9In;

  TimedFloat m_floatin0;
  InPort<TimedFloat> m_floatin0In;
  TimedFloat m_floatin1;
  InPort<TimedFloat> m_floatin1In;
  TimedFloat m_floatin2;
  InPort<TimedFloat> m_floatin2In;
  TimedFloat m_floatin3;
  InPort<TimedFloat> m_floatin3In;
  TimedFloat m_floatin4;
  InPort<TimedFloat> m_floatin4In;
  TimedFloat m_floatin5;
  InPort<TimedFloat> m_floatin5In;
  TimedFloat m_floatin6;
  InPort<TimedFloat> m_floatin6In;
  TimedFloat m_floatin7;
  InPort<TimedFloat> m_floatin7In;
  TimedFloat m_floatin8;
  InPort<TimedFloat> m_floatin8In;
  TimedFloat m_floatin9;
  InPort<TimedFloat> m_floatin9In;

  TimedDouble m_doublein0;
  InPort<TimedDouble> m_doublein0In;
  TimedDouble m_doublein1;
  InPort<TimedDouble> m_doublein1In;
  TimedDouble m_doublein2;
  InPort<TimedDouble> m_doublein2In;
  TimedDouble m_doublein3;
  InPort<TimedDouble> m_doublein3In;
  TimedDouble m_doublein4;
  InPort<TimedDouble> m_doublein4In;
  TimedDouble m_doublein5;
  InPort<TimedDouble> m_doublein5In;
  TimedDouble m_doublein6;
  InPort<TimedDouble> m_doublein6In;
  TimedDouble m_doublein7;
  InPort<TimedDouble> m_doublein7In;
  TimedDouble m_doublein8;
  InPort<TimedDouble> m_doublein8In;
  TimedDouble m_doublein9;
  InPort<TimedDouble> m_doublein9In;

  TimedBoolean m_boolin0;
  InPort<TimedBoolean> m_boolin0In;
  TimedBoolean m_boolin1;
  InPort<TimedBoolean> m_boolin1In;
  TimedBoolean m_boolin2;
  InPort<TimedBoolean> m_boolin2In;
  TimedBoolean m_boolin3;
  InPort<TimedBoolean> m_boolin3In;
  TimedBoolean m_boolin4;
  InPort<TimedBoolean> m_boolin4In;
  TimedBoolean m_boolin5;
  InPort<TimedBoolean> m_boolin5In;
  TimedBoolean m_boolin6;
  InPort<TimedBoolean> m_boolin6In;
  TimedBoolean m_boolin7;
  InPort<TimedBoolean> m_boolin7In;
  TimedBoolean m_boolin8;
  InPort<TimedBoolean> m_boolin8In;
  TimedBoolean m_boolin9;
  InPort<TimedBoolean> m_boolin9In;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedShortSeq m_sarrout;
  OutPort<TimedShortSeq> m_sarroutOut;
  TimedLongSeq m_larrout;
  OutPort<TimedLongSeq> m_larroutOut;
  TimedFloatSeq m_farrout;
  OutPort<TimedFloatSeq> m_farroutOut;
  TimedDoubleSeq m_darrout;
  OutPort<TimedDoubleSeq> m_darroutOut;
  TimedBooleanSeq m_barrout;
  OutPort<TimedBooleanSeq> m_barroutOut;

  int num, type;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_PrvPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  multiplexerSVC_impl m_multiplexer;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

};


extern "C"
{
  void MultiplexerInit(RTC::Manager* manager);
};

#endif // MULTIPLEXER_H
